from cement import Controller, ex
from cement import App
import requests
from termcolor import colored
from tabulate import tabulate
import unicodedata
import os
import yaml
from .utils import auth_error


class Project(Controller):
    class Meta:
        label = 'project'
        stacked_type = 'nested'
        stacked_on = 'base'
        description = 'Rogue Cli'
        epilog = 'Usage: rogue project command'
        help = 'Project'

        
        # arguments = [
        #     # list of tuples in the format `( [], {} )`
        #     ( [ '-n', '--name' ],
        #       { 'help' : 'name of project' } ),
        # ]



    # @ex(help="second-controller default command", hide=True)
    # def _default(self):
    #     print(self.app.pargs)
    #     print("Inside SecondController.default()")

    @ex(help='List all projects')
    def ls(self):
        try:
            token = self.app.config.get('rogue','token')
            url = self.app.config.get('rogue','url')
            r = requests.get(f'{url}/projects',headers={'Authorization': f'token {token}'})

            if r.status_code == 200:
                json_s = r.json()

                projects = list(map(lambda x: [x['name']], json_s))

                headers = ['name']

                self.app.render({'table':tabulate(projects, headers)},'table.jinja2')
            
            elif r.status_code == 401:
                auth_error(self.app)
            else:
                error = {
                    'msg': colored(r.json(),'red')   
                }
                self.app.render(error ,'msg.jinja2')

        except requests.exceptions.ConnectionError as err:
            self.app.render({'msg': colored('Erro api','red')} ,'msg.jinja2')

    @ex(help='create new project',
    arguments=[
        (['name'],
        {'help': 'project name'}),
        (['template'],
        {'help': 'template name'}),
        (['email'],
        {'help': 'owner email'})
    ]
    )
    def new(self):
        url = self.app.config.get('rogue','url')
        token = self.app.config.get('rogue','token')
        try:

            body = {
                "name": self.app.pargs.name,
                "template": self.app.pargs.template,
                "email": self.app.pargs.email
            }	

            r = requests.post(f'{url}/projects', json=body, headers={'Authorization': f'token {token}'})
            

            if r.status_code == 201:
                self.app.render(r.json(),'project.jinja2')
            elif r.status_code == 401:
                auth_error(self.app)
            else:
                error = {
                    'msg': colored(r.json()['message'],'red')   
                }
                self.app.render(error ,'msg.jinja2')
                # print(colored(r.json()['message'], 'red'))

        except requests.exceptions.ConnectionError as err:
            self.app.render({'msg': colored('Erro api','red')} ,'msg.jinja2')

    @ex(help='describe project',
    arguments=[
        (['name'],
        {'help': 'project name'}),
        (['-v', '--versions'],
        {'help': 'show versions of project','action':'store_true'}),
    ]
    )
    def describe(self):
        try:
            url = self.app.config.get('rogue','url')
            token = self.app.config.get('rogue','token')
            if self.app.pargs.versions:
                r = requests.get(f'{url}/projects/{self.app.pargs.name}/version',headers={'Authorization': f'token {token}'})

                if r.status_code == 200:

                    headers = ['version', 'published']
                    data = []
                    for j in r.json():
                        published = 'YES' if j['published'] else 'NO'
                        data.append([j['version'],published])

            
                    self.app.render({'table':tabulate(data, headers)},'table.jinja2')
                elif r.status_code == 401:
                    auth_error(self.app)
                else:
                    error = {
                        'msg': colored(r.json()['message'],'red')   
                    }
                    self.app.render(error ,'msg.jinja2')
                    
            
            else:
                print('No describe yet!')

        except requests.exceptions.ConnectionError as err:
            self.app.render({'msg': colored('Erro api','red')},'msg.jinja2')

    @ex(help='generate new version of project',
    arguments=[
        (['name'],
        {'help': 'project name'})
    ]
    )
    def new_version(self):
        try:
            url = self.app.config.get('rogue','url')
            token = self.app.config.get('rogue','token')

            r = requests.post(f'{url}/projects/{self.app.pargs.name}/version',headers={'Authorization': f'token {token}'})

            if r.status_code == 202:
                msg = {
                        'msg': colored('A new versions is generating...')   
                }
                self.app.render(msg ,'msg.jinja2')
            elif r.status_code == 401:
                auth_error(self.app)
            else:
                error = {
                        'msg': colored(r.json()['message'],'red')   
                }
                self.app.render(error ,'msg.jinja2')
                    
        except requests.exceptions.ConnectionError as err:
            self.app.render({'msg': colored('Erro api','red')},'msg.jinja2')


    @ex(help='publish project version',
    arguments=[
        (['name'],
        {'help': 'project name'}),
        (['version'],
        {'help': 'version project'}),
        (['extension'],
        {'help': 'extension of project', 'type': int}),
        (['-f', '--force'],
        {'help': 'override extension in use','action':'store_true'}),
    ]
    )
    def publish(self):
        url = self.app.config.get('rogue','url')
        token = self.app.config.get('rogue','token')
        try:

            # TODO: VALIDAR ANTES DE PUBLICAR
            arguments = self.app.pargs
            body = {
                        "extension": arguments.extension,
                        "override": arguments.force
                }

            r = requests.post(f'{url}/projects/{arguments.name}/{arguments.version}', json=body, headers={'Authorization': f'token {token}'})

            if r.status_code == 200:
                self.app.render({'msg': f'Version {arguments.version} published!'},'msg.jinja2')
            elif r.status_code == 401:
                auth_error(self.app)
            elif r.status_code == 409:
                self.app.render({'msg': colored(f"{r.json()['message']}, -f parameter can force",'yellow')},'msg.jinja2')
            else:
                self.app.render({'msg': colored(r.json(),'red')},'msg.jinja2')

        except requests.exceptions.ConnectionError as err:
            self.app.render({'msg': colored('Erro api','red')},'msg.jinja2')

    @ex(help='validate project',
    arguments=[
        (['name'],
        {'help': 'project name'}),
        (['-a', '--audios'],
        {'help': 'validate with audios','action':'store_true'}),
    ]
    )
    def validate(self):
        try:
            url = self.app.config.get('rogue','url')
            token = self.app.config.get('rogue','token')
            arguments = self.app.pargs
            body = {
                "validateAudios": arguments.audios
            }

            r = requests.post(f'{url}/projects/{arguments.name}/validate', json=body, headers={'Authorization': f'token {token}'})
            if r.status_code == 200:
                result = r.json()
                errors = 0
                warnings = 0
                obj = {
                    'states': []
                }

                for r in result:
                    state = {}
                    state['name'] = r
                    state['status'] = colored('OK', 'green')
                    state['messages'] = []
                    for s in result[r]['state']:
                        cell = f'[{s["cell"]}]' if 'cell' in s else ''
                        if s['type'] == 'error':
                            errors += 1
                            state['status'] = colored('NOK', 'red')
                            pre_txt = colored(f'[Error]{cell} ','red')
                        else:
                            warnings += 1
                            pre_txt = colored(f'[Warning]{cell} ','yellow')

                        state['messages'].append(f"{pre_txt}{s['message']}")
                    
                    for p in result[r]['prompts']:
                        cell = f'[{p["cell"]}]' if 'cell' in p else ''
                        if p['type'] == 'error':
                            errors += 1
                            state['status'] = colored('NOK', 'red')
                            pre_txt = colored(f'[Error]{cell} ','red')
                        else:
                            warnings +=1
                            pre_txt = colored(f'[Warning]{cell} ','yellow')

                        state['messages'].append(f"{pre_txt}{p['message']}")

                    for t in result[r]['transitions']:
                        cell = f'[{t["cell"]}]' if 'cell' in t else ''
                        if t['type'] == 'error':
                            errors += 1
                            state['status'] = colored('NOK', 'red')
                            pre_txt = colored(f'[Error]{cell} ','red')
                        else:
                            warnings +=1
                            pre_txt = colored(f'[Warning]{cell} ','yellow')


                        state['messages'].append(f"{pre_txt}{t['message']}{cell}") 

                    obj['states'].append(state)

                obj['errors'] = errors
                obj['warnings'] = warnings
                
                self.app.render(obj,'validate.jinja2')           
            elif r.status_code == 401:
                auth_error(self.app)
            else:
                self.app.render({'msg': colored(r.json(),'red')},'msg.jinja2')
        except requests.exceptions.ConnectionError as err:
            self.app.render({'msg': colored('Erro api','red')},'msg.jinja2')
        except Exception as err:
            self.app.render({'msg': colored('Erro api','red')},'msg.jinja2')

    @ex(help='get schema for integration',
    arguments=[
        (['name'],
        {'help': 'project name'})
    ]
    )
    def schema(self):
        try:
            url = self.app.config.get('rogue','url')
            token = self.app.config.get('rogue','token')
            arguments = self.app.pargs
            r = requests.get(f'{url}/projects/{arguments.name}/schema',headers={'Authorization': f'token {token}'})

            if r.status_code == 200:
                self.app.render({'msg': r.text.replace(r'\n', '\n').replace('"','')},'msg.jinja2')
            elif r.status_code == 401:
                auth_error(self.app)
            else:
                self.app.render({'msg': colored(r.json(),'red')},'msg.jinja2')
        except requests.exceptions.ConnectionError as err:
            self.app.render({'msg': colored('Erro api','red')},'msg.jinja2')
    
    @ex(help='get prompts from project and save as yaml file',
    arguments=[
        (['name'],
        {'help': 'project name'}),
        (['output_path'],
        {'help': 'output path'})
    ]
    )
    def prompts(self):
        try:
            url = self.app.config.get('rogue','url')
            token = self.app.config.get('rogue','token')
            arguments = self.app.pargs
            r = requests.get(f'{url}/projects/{arguments.name}/prompts',headers={'Authorization': f'token {token}'})
           
            if r.status_code == 200:
                file_name = f'{arguments.output_path.rstrip("/")}/{arguments.name}-prompts.yml'

                data = r.json()

                for d in data:
                    data[d] = self.removeDiacritic(data[d])


                f = open(file_name, 'w')
                yaml.dump({'prompts': data}, f, default_flow_style=False)
                f.close()

                self.app.render({'msg': f'File {file_name} created'},'msg.jinja2')
            elif r.status_code == 401:
                auth_error(self.app)
            else:
                self.app.render({'msg': colored(r.json(),'red')},'msg.jinja2')

        except requests.exceptions.ConnectionError as err:
            self.app.render({'msg': colored('Erro api','red')},'msg.jinja2')
        except Exception as err:
            self.app.render({'msg': colored(err,'red')},'msg.jinja2')

    @ex(help='transfer Audios to Behavior-Bot',
    arguments=[
        (['name'],
        {'help': 'project name'})
    ]
    )
    def transfer_audios(self):
        try:
            url = self.app.config.get('rogue','url')
            token = self.app.config.get('rogue','token')
            arguments = self.app.pargs

            r = requests.post(f'{url}/projects/{arguments.name}/transfer',headers={'Authorization': f'token {token}'})
           
            if r.status_code == 200:
                self.app.render({'msg': f'Transfered'},'msg.jinja2')
            elif r.status_code == 401:
                auth_error(self.app)
            else:
                self.app.render({'msg': colored(r.json(),'red')},'msg.jinja2')

        except requests.exceptions.ConnectionError as err:
            self.app.render({'msg': colored('Erro api','red')},'msg.jinja2')
        except Exception as err:
            self.app.render({'msg': colored(err,'red')},'msg.jinja2')




    def removeDiacritic(self,text):
        return unicodedata.normalize('NFKD', text).encode('ASCII', 'ignore').decode("utf-8")
     
        


    



from cement import Controller, ex
from cement.utils.version import get_version_banner
from ..core.version import get_version
from pathlib import Path
import webbrowser
import requests
import uuid
import yaml
import time

VERSION_BANNER = """
Rogue Cli %s
%s
""" % (get_version(), get_version_banner())


class Base(Controller):
    class Meta:
        label = 'base'

        # text displayed at the top of --help output
        description = 'Rogue Cli'

        # text displayed at the bottom of --help output
        epilog = 'Usage: rogue command1 --foo bar'

        # controller level arguments. ex: 'rogue --version'
        arguments = [
            ### add a version banner
            ( [ '-v', '--version' ],
              { 'action'  : 'version',
                'version' : VERSION_BANNER } ),
        ]


    def _default(self):
        """Default action if no sub-command is passed."""

        self.app.args.print_help()


    @ex(
    help='login'
    )
    def login(self):
        url = self.app.config.get('rogue','url')
        home = str(Path.home())
        token = str(uuid.uuid4())
        old_conf = self.app.config.get_dict()['rogue']
        
        new_conf = {
            'rogue': old_conf
        }

        new_conf['rogue']['token'] = token    

        with open(f'{home}/.rogue.conf', 'w') as outfile:
                yaml.dump(new_conf, stream=outfile, default_flow_style=False, indent=3)
        
        login_url = f'{url}/accounts/login?token={token}'
        browser = webbrowser.open(login_url)

        if not browser:
            self.app.render({'msg':f'You need to login: {login_url}'} ,'msg.jinja2')

        while True:
            
            r = requests.post(f'{url}/accounts/session', headers={'Authorization': f'token {token}'})
            print(r.json())
            if r.status_code == 200:
                self.app.render({'msg':f'You need to login: {login_url}'} ,'msg.jinja2')
                break
            else:
                time.sleep(5)
            




from cement import App, TestApp, init_defaults
from cement.core.exc import CaughtSignal
from .core.exc import RogueError
from .controllers.base import Base
from .controllers.project import Project
from .controllers.engine import Engine
from pathlib import Path
import os
import yaml

# configuration defaults
CONFIG = init_defaults('rogue', 'log.logging')
CONFIG['rogue']['url'] = 'https://dev.admin2.rogue.mutantcxo.com'
CONFIG['log.logging']['level'] = 'info'


class Rogue(App):
    """Rogue Cli primary application."""

    class Meta:
        label = 'rogue'

        # configuration defaults
        config_defaults = CONFIG

        # call sys.exit() on close
        exit_on_close = True

        # load additional framework extensions
        extensions = [
            'yaml',
            'colorlog',
            'json',
            'jinja2',
            'tabulate'
        ]

        # configuration handler
        config_handler = 'yaml'

        home = str(Path.home())
       
        if not os.path.isfile(f'{home}/.rogue.conf'):
            conf = {'rogue': {
                'token': '',
                 'url': 'https://dev.admin2.rogue.mutantcxo.com'
            }}

            with open(f'{home}/.rogue.conf', 'w') as outfile:
                yaml.dump(conf, stream=outfile, default_flow_style=False, indent=3)

        config_files = [
            '~/.rogue.conf'
        ]

        # configuration file suffix
        config_file_suffix = '.yml'

        # set the log handler
        log_handler = 'colorlog'

        # set the output handler
        output_handler = 'jinja2'

        # register handlers
        handlers = [
            Base,
            Project,
            Engine
        ]


class RogueTest(TestApp,Rogue):
    """A sub-class of Rogue that is better suited for testing."""

    class Meta:
        label = 'rogue'


def main():
    with Rogue() as app:
        try:
            app.run()

        except AssertionError as e:
            print('AssertionError > %s' % e.args[0])
            app.exit_code = 1

            if app.debug is True:
                import traceback
                traceback.print_exc()

        except RogueError as e:
            print('RogueError > %s' % e.args[0])
            app.exit_code = 1

            if app.debug is True:
                import traceback
                traceback.print_exc()

        except CaughtSignal as e:
            # Default Cement signals are SIGINT and SIGTERM, exit 0 (non-error)
            print('\n%s' % e)
            app.exit_code = 0


if __name__ == '__main__':
    main()



from  termcolor import colored

def auth_error(app):
    token = app.config.get('rogue','token')
    if token:
        msg = colored('Session expired please execute login again','red')
    else:
        msg = colored('You need execute login first','red')

    app.render({'msg': msg } ,'msg.jinja2')
    return False


from cement import Controller, ex
from cement import App
import requests
from termcolor import colored
from tabulate import tabulate
from .utils import auth_error

class Engine(Controller):
    class Meta:
        label = 'engine'
        stacked_type = 'nested'
        stacked_on = 'base'
        description = 'Rogue Cli'
        epilog = 'Usage: rogue engine command'


    @ex(help='show all running projects on rogue')
    def running(self):
        url = self.app.config.get('rogue','url')
        token = self.app.config.get('rogue','token')

        try:
            r = requests.get(f'{url}/projects/running',  headers={'Authorization': f'token {token}'})

            if r.status_code == 200:
                headers = ['Extension', 'Name', 'version']
                result = r.json()
                dev = result['dev']
                prd = result['prd']
                data = []

                for d in dev:
                    data.append([d,dev[d]['name'], 'dev'])

                for p in prd:
                    data.append([p,prd[p]['name'], prd[p]['version']])

                
                self.app.render({'table':tabulate(data, headers)},'table.jinja2')
            elif r.status_code == 401:
                auth_error(self.app)

        except requests.exceptions.ConnectionError as err:
            print(colored('A error on rogue-dialog-api ocurred','red'))

    @ex(help='show all running projects on rogue')
    def configure(self):
        # lala = input('oioi')
        # self.app.config.set('rogue','blah', True)
        # other_config = dict()
        # other_config['myapp'] = dict()
        # other_config['myapp']['foo'] = 'not bar'
        # self.app.config.merge(other_config)
        print(self.app.config.get('rogue','name'))




from setuptools import setup, find_packages
from rogue.core.version import get_version

VERSION = get_version()

f = open('README.md', 'r')
LONG_DESCRIPTION = f.read()
f.close()

requirements = [
'cement==3.0.4',
'jinja2',
'pyyaml',
'colorlog',
'pytest',
'pytest-cov',
'coverage',
'twine>=1.11.0',
'setuptools>=38.6.0',
'wheel>=0.31.0',
'requests>=2.22.0',
'termcolor>=1.1.0',
'tabulate>=0.8.7'
]

setup(
    name='mutant-rogue',
    version=VERSION,
    description='Rogue Cli',
    long_description=LONG_DESCRIPTION,
    long_description_content_type='text/markdown',
    author='Pedrinho',
    author_email='pedro.carvalho@mutantbr.com',
    url='https://bitbucket.org/unearsa/rogue-cli',
    license='unlicensed',
    packages=find_packages(exclude=['ez_setup', 'tests*']),
    package_data={'mutant-rogue': ['templates/*']},
    include_package_data=True,
    python_requires='>=3.6',
    install_requires=requirements,
    entry_points="""
        [console_scripts]
        rogue = rogue.main:main
    """,
)
